import altair as alt
import pandas as pd

def corrplot(df, print_corr=True, shape="circle"):
    """Generates a correlation plot for the numeric variables
    in a dataframe, and prints the correlation values.

    Parameters
    ----------
    df : DataFrame
        The dataframe containing the data to be plotted.
    print_corr : boolean
        Determines whether the function will print the
        correlation values after the graph or not.
    shape : string
        Determines the shape to use in the graph to represent
        the value of the correlation. Its possible values 
        are "circle" and "square".

    Returns
    -------
    altair.vegalite.v4.api.Chart
        Correlation plot between the numerical variables
        of a dataframe.

    Examples
    --------
    >>> from magmaviz.magmaviz import corrplot
    >>> corrplot(movies_data, print_corr=True, shape="square")
    """

    # Checking that the dataframe is a pandas dataframe type
    if not isinstance(df, pd.core.frame.DataFrame):
        raise TypeError(
            "'df' should be of type 'pandas.core.frame.DataFrame', a pandas dataframe."
        )
    
    # Checking that type for print_corr is a boolean
    if not isinstance(print_corr, bool):
        raise TypeError("'print_corr' should be of type 'boolean'.")
        
    # Checking that type of shape is string
    if not isinstance(shape, str):
        raise TypeError("'shape' should be of type 'str'.")

    # Checking if shape has a valid value
    if shape not in ["circle", "square"]:
        raise ValueError("The value specified for 'shape' is not valid. It should be 'circle' or 'square'")

    # Create the correlation dataframe
    corr_df = (
        df
        .select_dtypes('number')
        .corr('spearman')
        .stack()                            # Get df into long format for altair
        .reset_index(name='Correlation'))   # Name the index that is reset to "Correlation"

    # Set cases where the variable is compared vs itself to zero
    corr_df.loc[corr_df['Correlation'] == 1, 'Correlation'] = 0

    # Add a column with the absolute value
    corr_df['abs'] = corr_df['Correlation'].abs()

    # Rename the columns
    corr_df.rename(columns={'level_0': 'Variable1', 'level_1': 'Variable2'}, inplace=True)
    
    # Creating the plot with circles
    if shape == "circle":
        chart = alt.Chart(corr_df).mark_circle().encode(
                    x='Variable1',
                    y='Variable2',
                    size='abs',
                    color=alt.Color('Correlation', scale=alt.Scale(scheme='purpleorange', domain=(-1, 1))))
    
    # Creating the plot with squares
    if shape == "square":
        chart = alt.Chart(corr_df).mark_square().encode(
                    x='Variable1',
                    y='Variable2',
                    size='abs',
                    color=alt.Color('Correlation', scale=alt.Scale(scheme='purpleorange', domain=(-1, 1))))
    
    # Printing the correlations without repeating the combination of variables
    if print_corr:
        print_corr = corr_df.copy()
        print_corr['Variable12'] = print_corr['Variable1'] + print_corr['Variable2']
        print_corr['Variable21'] = print_corr['Variable2'] + print_corr['Variable1']

        for value in print_corr['Variable21']:
            if value in list(print_corr['Variable12']):
                print_corr.drop(print_corr[print_corr.Variable21 == value].index, inplace=True)

        print_corr.reset_index(drop=True, inplace=True)
        print(print_corr[['Variable1', 'Variable2', 'Correlation']])
    
    return chart


import altair as alt
import pandas as pd
import re
from pandas.api.types import is_string_dtype
from pandas.api.types import is_numeric_dtype

def scatterplot(df, x, y, c="", t="", o=0.5, s=50, xtitle="", ytitle="", ctitle="", xzero=False, yzero=False, shapes=True):
    """Plot a scatterplot on the dataframe with the magma color scheme.
    
    Parameters
    ----------
    df : dataframe
         Dataframe containing the numerical features x and y
    x : string
        Column-name of the numerical variable to be plotted on the x-axis
    y : string
        Column-name of the numerical variable to be plotted on the y-axis
    c : string
        Column-name of the categorical variable to color-code the data points
        Default value is blank for cases when there is no categorical column
    t : string
        Title of the plot. Default value is blank. If not provided,
        title will be computed based on x, y and/or c
    o : float
        Opacity of the data points
        Default value is 0.5
    s : integer
        Size of the data points
        Default value is 50
    xtitle : string
             Title of the x-axis. Default value is blank. If not provided,
             title will be proper case of the x axis column
    ytitle : string
             Title of the y-axis. Default value is blank. If not provided,
             title will be proper case of the y axis column
    ctitle : string
             Title of the color legend. Default value is blank. If not provided,
             title will be proper case of the color column
    xzero : boolean
            Scale the x-axis to start from 0 by specifying True
            Default value is set to False
    yzero : boolean
            Scale the y-axis to start from 0 by specifying True
            Default value is set to False
    shapes : boolean
             Assign the color column to the shape attribute of the plot if True
             Default value is set to True

    Returns
    -------
    altair.vegalite.v4.api.Chart
        Scatterplot between the numerical variables x and y

    Example
    -------
    >>> from magmaviz.scatterplot import scatterplot
    >>> from vega_datasets import data
    >>> scatterplot(data.iris(), "sepalLength", "sepalWidth", "species",
                    "Iris Sepal Length vs Sepal Width across Species",
                    1.0, 50, "Sepal Length", "Sepal Width", "", False, False, True)
    """

    # Basic checks to see if parameters passed to function call contain expected values
    # check if the dataframe is a pandas dataframe
    if not isinstance(df, pd.core.frame.DataFrame):
        raise TypeError("'df' should be of type 'pandas.core.frame.DataFrame', a pandas dataframe.")

    # check if column name for x-axis is a string
    if not isinstance(x, str):
        raise TypeError("Invalid value passed to 'x' axis: Assign column name as a 'string'.")

    # check if column name for y-axis is a string
    if not isinstance(y, str):
        raise TypeError("Invalid value passed to 'y' axis: Assign column name as a 'string'.")

    # check if column name for color is a string
    if not isinstance(c, str):
        raise TypeError("Invalid value passed to 'color' variable: Assign column name as a 'string'.")

    # check if title is a string
    if not isinstance(t, str):
        raise TypeError("Invalid value passed to 't' variable: Assign title as a 'string'.")

    # check if opacity is a number
    if not isinstance(o, float):
        raise TypeError("Invalid value passed to 'o' variable: Assign opacity value as a decimal between 0 and 1.")

    # check if size is an integer
    if not isinstance(s, int):
        raise TypeError("Invalid value passed to 's' variable: Assign size value as an integer between 1 and 100.")

    # check if x-axis title is a string
    if not isinstance(xtitle, str):
        raise TypeError("Invalid value passed to 'xtitle' variable: Assign x-axis title as a 'string'.")

    # check if y-axis title is a string
    if not isinstance(ytitle, str):
        raise TypeError("Invalid value passed to 'ytitle' variable: Assign y-axis title as a 'string'.")

    # check if color legend title is a string
    if not isinstance(ctitle, str):
        raise TypeError("Invalid value passed to 'ctitle' variable: Assign color legend title as a 'string'.")

    # check if xzero is a boolean
    if not isinstance(xzero, bool):
        raise TypeError("Invalid value passed to 'xzero' variable: Assign boolean True to begin x axis from zero.")

    # check if yzero is a boolean
    if not isinstance(yzero, bool):
        raise TypeError("Invalid value passed to 'ctitle' variable: Assign boolean True to begin y axis from zero.")

    # check if shapes is a boolean
    if not isinstance(shapes, bool):
        raise TypeError("Invalid value passed to 'shapes' variable: Assign boolean True to show different shapes for each color category.")

    # Advanced checks to see whether columns exists, opacity and size are within expected range, etc.
    # check if column name assigned to x-axis is present in the dataframe
    assert x in list(
        df.columns
    ), "The column specified for 'x' axis does not exist in the dataframe."

    # check if column name assigned to y-axis is present in the dataframe
    assert y in list(
        df.columns
    ), "The column specified for 'y' axis does not exist in the dataframe."

    # check if column name assigned to color is present in the dataframe
    if c != "":
        assert c in list(
            df.columns
        ), "The column specified for 'color' does not exist in the dataframe."

    # check if opacity value is in the range 0.1 to 1.0
    if o < 0.1 or o > 1.0:
        raise TypeError("Opacity value must be in the range [0.1, 1.0]")

    # check if size value is in the range 1 to 100
    if s < 1 or s > 100:
        raise TypeError("Size of data points must be in the range [1, 100]")

    # check if x-axis column is numeric or not
    assert is_numeric_dtype(df[x]), "The column assigned to 'x' axis is not of type numeric."

    # check if y-axis column is numeric or not
    assert is_numeric_dtype(df[y]), "The column assigned to 'y' axis is not of type numeric."

    # check if color column is string or not
    if c != "":
        assert is_string_dtype(df[c]), "The column assigned to 'color' is not of type string."

    # add proper titles to axes, legend and plot
    if xtitle == "":
        xtitle = re.sub(r"[_.,-]", " ", x)
    if ytitle == "":
        ytitle = re.sub(r"[_.,-]", " ", y)
    if ctitle == "":
        ctitle = re.sub(r"[_.,-]", " ", c)
    if t == "":
        if c == "":
            t = f"{xtitle.title()} vs {ytitle.title()}"
        else:
            t = f"{xtitle.title()} vs {ytitle.title()} by {ctitle.title()}"

    # scatterplot
    if c == "":
        plot = alt.Chart(
            data=df, title=t
        ).mark_point(
            opacity=o, size=s, color="#721F81"
        ).encode(
            alt.X(x, title=xtitle.capitalize(), scale=alt.Scale(zero=xzero)),
            alt.Y(y, title=ytitle.capitalize(), scale=alt.Scale(zero=yzero))
        )
    else:
        if shapes is False:
            plot = alt.Chart(
                data=df, title=t
            ).mark_point(
                opacity=o, size=s
            ).encode(
                alt.X(x, title=xtitle.capitalize(), scale=alt.Scale(zero=xzero)),
                alt.Y(y, title=ytitle.capitalize(), scale=alt.Scale(zero=yzero)),
                alt.Color(c, title=ctitle.capitalize(), scale=alt.Scale(scheme="magma"))
            )
        else:
            plot = alt.Chart(
                data=df, title=t
            ).mark_point(
                opacity=o, size=s
            ).encode(
                alt.X(x, title=xtitle.capitalize(), scale=alt.Scale(zero=xzero)),
                alt.Y(y, title=ytitle.capitalize(), scale=alt.Scale(zero=yzero)),
                alt.Color(c, title=ctitle.capitalize(), scale=alt.Scale(scheme="magma")),
                shape=c
            )

    return plot


import altair as alt
import pandas as pd
import numpy as np
import re


def boxplot(df, x, y, facet=False):
    """Plot a boxplot with the magma color scheme and an option to facet.

    Parameters
    ----------
    df : dataframe
        Dataframe containing the variables for plotting
    x : string
        Column name of the numerical variable to view the distribution of
    y : list
        Column name containing the categorical variables to assign boxes to
    facet : boolean
        Determines whether separate graphs will be created for each category

    Returns
    -------
    altair.vegalite.v4.api.Chart
        Boxplot displaying distribution of categorical variables with/without faceting

    Examples
    --------
    >>> from magmaviz.magmaviz import boxplot
    >>> boxplot(cars, "Miles_per_Gallon", "Origin", facet=True)
    """

    # checking that the dataframe is a pandas dataframe type
    if not isinstance(df, pd.core.frame.DataFrame):
        raise TypeError(
            "'df' should be of type 'pandas.core.frame.DataFrame', a pandas dataframe."
        )
    # checking that type of column name for x is a string
    if not isinstance(x, str):
        raise TypeError("'x' should be of type 'str'.")
    # checking that type of column name for y is a string
    if not isinstance(y, str):
        raise TypeError("'y' should be of type 'str'.")
    # checking that type for facet is a boolean
    if not isinstance(facet, bool):
        raise TypeError("'facet' should be of type 'boolean'.")

    # checking if x is a column name in the dataframe
    assert x in list(
        df.columns
    ), "This column specified for 'x' does not exist in the dataframe."
    # checking if y is a column name in the dataframe
    assert y in list(
        df.columns
    ), "This column specified for 'y' does not exist in the dataframe."

    # creating titles
    x_title = re.sub(r"[_.,-]", " ", x)

    y_title = re.sub(r"[_.,-]", " ", y)

    # creating plot object
    plot = (
        alt.Chart(df)
        .mark_boxplot()
        .encode(
            x=alt.X(x, title=x_title),
            y=alt.Y(y, title=y_title),
            color=alt.Color(y, scale=alt.Scale(scheme="magma"), legend=None),
        )
    )

    # facet if required
    if facet == True:
        return plot.encode(x=alt.X(x, title=None), y=alt.Y(y, title=None)).facet(row=y)

    else:
        return plot



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['magmaviz']

package_data = \
{'': ['*']}

install_requires = \
['altair>=4.2.0,<5.0.0', 'pandas>=1.3.5,<2.0.0', 'vega-datasets>=0.9.0,<0.10.0']

setup_kwargs = {
    'name': 'magmaviz',
    'version': '0.3.2',
    'description': 'This package contains four different data visualization functions with the magma theme for EDA.',
    'long_description': '# magmaviz\n\n![example workflow](https://github.com/UBC-MDS/magmaviz/actions/workflows/ci-cd.yml/badge.svg)\n\nExploratory Data Analysis is one of the key steps in a machine learning project. This package aims to make this process easy by providing python functions based on the \'Altair\' package to plot four common types of plots with the magma color scheme. To maximize interpretability, the plots have defined color schemes (discrete, diverging, sequential) based on the kind of data they show.\n\n## Installation\n\n```bash\n$ pip install -i https://test.pypi.org/simple/ magmaviz\n```\n\n## Usage \n\nThe interactive version of the usage section can be found here: \n\n[![Documentation Status](https://readthedocs.org/projects/magmaviz/badge/?version=latest)](https://magmaviz.readthedocs.io/en/latest/?badge=latest)\n\n\nThis package defines four data visualization functions, all with a magma color scheme. They are meant to be used in any data analysis projects using Python. \n\n### Boxplot\n\nReturns a boxplot based on a data frame, a numerical feature to view the distribution of and a categorical feature to bucket data into categories. Additionally, there is a boolean option to facet the boxplots into separate charts.\n\n```python\nfrom magmaviz.boxplot import boxplot\nboxplot(df, x, y, facet=False)\n```\n\n### Correlation Plot\n\nReturns a correlation plot based on the numerical features present in the data frame. While the default plot would use circle shapes, an auxiliary input provides the flexibility to switch to square shapes. Additionally, it will print the correlated numerical feature pairs along with their correlation values.\n\n```python\nfrom magmaviz.corrplot import corrplot\ncorrplot(df, print_corr=True, shape="square")\n```\n\n### Histogram\n\nReturns a histogram based on the data frame and a categorical feature to plot on the x-axis. The y-axis will display the result of some of the following aggregating functions:\n- Average\n- Count\n- Distinct\n- Max\n- Min\n- Median\n- Mean\n- Among others (listed in documentation for the function).\n\n```python\nfrom magmaviz.histogram import histogram\nhistogram(mtcars, "cars", "count()")\n```\n\n### Scatterplot\n\nReturns a scatterplot based on the data frame and two numerical feature names passed as the required inputs. There are auxiliary inputs that provide the flexibility to:\n- Color code or change the shape of the data points on a categorical variable\n- Set a title to the plot, x-axis, y-axis and color legend\n- Change the opacity and size of the data points\n- Set the scale of the x-axis and y-axis to start from zero\n\n```python\nfrom magmaviz.scatterplot import scatterplot\nscatterplot(df, x, y, c="", t="", o=1.0, s=50, xtitle="", ytitle="", ctitle="", xzero=False, yzero=False, shapes=True)\n```\n\n### Fit within the Python ecosystem\n\nOur package will build onto the existing features of \'Altair\' using the magma color scheme. It serves as an automated plotter and is a higher level implementation of it. Essentially it circumvents the need to code every single detail and allows the user to focus on the output. We came across two packages that have a similar line of thought:\n\n- [deneb](https://pypi.org/project/deneb/) (Altair) - uses the same base as this package\n- [spartan-viz](https://pypi.org/project/spartan-viz/) (Matplotlib) - same philosophy as this package: focus on good use of color\n\n\n## Contributing\n\nThe primary contributors to this package are:\n\n1. Abdul Moid Mohammed\n2. Mukund Iyer\n3. Irene Yan\n4. Rubén De la Garza Macías\n\nWe welcome new ideas and contributions. Please refer to the contributing guidelines in the CONTRIBUTING.MD file. Do note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`magmaviz` was created by Abdul Moid Mohammed, Mukund Iyer, Irene Yan, Rubén De la Garza Macías. It is licensed under the terms of the MIT license.\n\n## Credits\n\n`magmaviz` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n',
    'author': 'Abdul Moid Mohammed, Mukund Iyer, Irene Yan, Rubén De la Garza Macías',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import altair as alt
import pandas as pd

def histogram(df, x, y):
    """Plot a histogram with the magma color scheme

    Parameters
    ----------
    df : dataframe
        Dataframe containing the variables for plotting
    x : string
        Column name of the variable to be plotted on the x-axis
    y : string
        An aggregation function to be plotted on the y-axis.
        The supported aggregation operations are: ['average', 'count',
        'distinct', 'max', 'mean', 'median', 'min', 'missing', 'product',
        'q1', 'q3', 'ci0', 'ci1', 'stderr', 'stdev', 'stdevp', 'sum', 
        'valid', 'values', 'variance', 'variancep']

    Returns
    -------
    altair.vegalite.v4.api.Chart
        A histogram displaying distribution based on the aggregation function

    Examples
    --------
    >>> from magmaviz.histogram import histogram
    >>> histogram(mtcars, "cars", "count()")
    """
    # check if the type of df is a pandas DataFrame
    if not isinstance(df, pd.core.frame.DataFrame):
        raise TypeError(
            "'df' should be of type 'pandas.core.frame.DataFrame', a pandas dataframe."
        )
        
    # check if the type of x is a string
    if not isinstance(x, str):
        raise TypeError("'x' should be of type 'str'.")
              
    # check if x is a column in the dataframe
    if x not in list(df.columns):
        raise ValueError("The column specified for 'x' does not exist in the dataframe.")
    
    validate(df, y)
  
    chart = alt.Chart(df).mark_bar().encode(
        x=x,
        y=y,
        color=alt.Color(y, scale=alt.Scale(scheme="magma"))
    )
    return chart

def validate(df, y):
    """A helper function to validate input y
    Parameters
    ----------
    df : dataframe
        The input dataframe
    y : string
        The user specified aggregation function to be plotted on the y-axis.
        The supported aggregation operations are: ['average', 'count',
        'distinct', 'max', 'mean', 'median', 'min', 'missing', 'product',
        'q1', 'q3', 'ci0', 'ci1', 'stderr', 'stdev', 'stdevp', 'sum', 
        'valid', 'values', 'variance', 'variancep']
    Returns
    -------
        This function only raises errors
    Examples
    --------
    >>> validate(data.iris(), 'mean(petalLength)')
    >>> validate(data.iris(), 'count()')
    """
    supported_operations = ['average', 'count',
        'distinct', 'max', 'mean', 'median', 'min', 'missing', 'product',
        'q1', 'q3', 'ci0', 'ci1', 'stderr', 'stdev', 'stdevp', 'sum', 
        'valid', 'values', 'variance', 'variancep']
    # check if the type of y is a string
    if not isinstance(y, str):
        raise TypeError("'y' should be of type 'str'.")
  
    splited_y = y.split("(")
    function_name = splited_y[0]
    
    if len(splited_y) != 2:
        raise ValueError("'y' is not in a correct format as an aggregation function.")
    
    if function_name not in supported_operations:
        raise ValueError(
            "The aggregation function specified for 'y' " +
            "is not one of " + str(supported_operations))
        
    encoding_field = splited_y[1].split(')')[0]
    
    # For aggregation functions that need an encoding field, check if the input
    #   has one and is a valid column
    if function_name in ['average', 'distinct', 'max', 'mean', 
                         'median', 'min', 'missing', 'product',
                         'q1', 'q3', 'ci0', 'ci1', 'stderr', 'stdev', 
                         'stdevp', 'sum', 'valid', 'values', 'variance',
                         'variancep']:
        if encoding_field == '':
            raise ValueError(
            "The encoding field 'y' is specified without a type; " +
            "the type cannot be inferred because it does not match any column in the data"
            )
        
        if encoding_field not in list(df.columns):
            raise ValueError("The encoding field for 'y' does not exist in the dataframe.")